NoDuelData = {};

NoDuelData.whisper = NODUEL_DEFAULTWHISPER;
NoDuelData.minimappos = 45;
NoDuelData.duelok = nil;
NoDuelData.hidebutton = nil;

local ldb = LibStub:GetLibrary("LibDataBroker-1.1");

local dataobj = ldb:NewDataObject("NoDuel", {
    type = "launcher",
    icon = "Interface\\Addons\\NoDuel\\icons\\duelok-dataobject",
    OnClick = function(self, button)
        if button == "LeftButton" then
            NoDuel_MinimapButton_OnClick();
        end
    end,
    OnTooltipShow = function(tooltip)
        NoDuel_CreateTooltip(tooltip);
    end
});

-- functions
function NoDuel_OnLoad(self)
    self:RegisterEvent("ADDON_LOADED");
    self:RegisterEvent("PLAYER_ENTERING_WORLD");

    self:RegisterEvent("DUEL_REQUESTED");
    self:RegisterEvent("PET_BATTLE_PVP_DUEL_REQUESTED");

    SLASH_NODUEL1 = "/noduel";
    SlashCmdList["NODUEL"] = function(msg)
        NoDuel_SlashCommandHandler(msg);
    end

    NoDuel_CheckButtonVisibility();
end

function NoDuel_OnEvent(self, event, ...)
    if (event == "ADDON_LOADED") then

        local addonName = ...;
        if (addonName == "NoDuel") then
            if (not NoDuelData.minimappos) then
                NoDuelData.minimappos = 45;
            end

            NoDuel_MinimapButton_Reposition();
            NoDuel_MinimapButton_InitButton();
        end
    end
    if (event == "DUEL_REQUESTED" or event == "PET_BATTLE_PVP_DUEL_REQUESTED") then
        local playerName = ...;
        NoDuel_CheckDuelRequested(playerName);
    end
end

function NoDuel_SlashCommandHandler(msg)
    if (msg == "reload") then
        ReloadUI();
    elseif (string.find(msg, "whisper") == 1) then
        local whisper = string.sub(msg, 9);
        NoDuel_SetWhisper(whisper);
    elseif (msg == "togglebutton") then
        NoDuel_ToggleButton();
    end
end

function NoDuel_CheckButtonVisibility()
    if (NoDuelData.hidebutton == 1) then
        NoDuel_MinimapButton:Hide();
    else
        NoDuel_MinimapButton:Show();
    end
end

function NoDuel_ToggleButton()
    if (NoDuelData.hidebutton == 1) then
        DEFAULT_CHAT_FRAME:AddMessage("NoDuel: show minimap button", 1.0, 1.0, 1.0, 1, 10);
        NoDuelData.hidebutton = nil;
    else
        DEFAULT_CHAT_FRAME:AddMessage("NoDuel: hide minimap button", 1.0, 1.0, 1.0, 1, 10);
        NoDuelData.hidebutton = 1;
    end

    NoDuel_CheckButtonVisibility();
end

function NoDuel_GetWhisper()
    if (not NoDuelData.whisper) then
        NoDuelData.whisper = NODUEL_DEFAULTWHISPER;
    end

    return NoDuelData.whisper;
end

function NoDuel_SetWhisper(whisper)
    DEFAULT_CHAT_FRAME:AddMessage("NoDuel whisper set to: "..whisper, 1.0, 1.0, 1.0, 1, 10);
    NoDuelData.whisper = whisper;
end

function NoDuel_CheckDuelRequested(playerName)
    if (not NoDuelData.duelok) then
        CancelDuel();

        StaticPopup_Hide("DUEL_REQUESTED");
        StaticPopup_Hide("PET_BATTLE_PVP_DUEL_REQUESTED");
        
        local message = NoDuel_GetWhisper();

        if (strlen(message) > 0) then
            -- in cross-realm battlegrounds, the format "Name-Realm" can be used to target a player from another realm
            SendChatMessage(NoDuel_GetWhisper(), "WHISPER", nil, playerName);
        end
    end
end

function NoDuel_MinimapButton_Reposition()
    NoDuel_MinimapButton:SetPoint("TOPLEFT", "Minimap", "TOPLEFT",
        52-(80*cos(NoDuelData.minimappos)), (80*sin(NoDuelData.minimappos))-52);
end

function NoDuel_MinimapButton_DraggingFrame_OnUpdate()

    local xpos,ypos = GetCursorPosition();
    local xmin,ymin = Minimap:GetLeft(), Minimap:GetBottom();

    xpos = xmin-xpos/UIParent:GetScale()+70;
    ypos = ypos/UIParent:GetScale()-ymin-70;

    NoDuelData.minimappos = math.deg(math.atan2(ypos,xpos));
    NoDuel_MinimapButton_Reposition();
end

function NoDuel_MinimapButton_OnClick()
    if (not NoDuelData.duelok) then
        NoDuelData.duelok = 1;
    else
        NoDuelData.duelok = nil;
    end

    NoDuel_MinimapButton_InitButton();

    if (NoDuelData.duelok) then
        DEFAULT_CHAT_FRAME:AddMessage(NODUEL_ENABLED, 1.0, 1.0, 1.0, 1, 10);
    else
        DEFAULT_CHAT_FRAME:AddMessage(NODUEL_DISABLED, 1.0, 1.0, 1.0, 1, 10);
    end
end

function NoDuel_MinimapButton_InitButton()
    if (NoDuelData.duelok) then
        NoDuel_MinimapButton_Icon:SetTexture("Interface\\Addons\\NoDuel\\icons\\duelok");
        dataobj.icon = "Interface\\Addons\\NoDuel\\icons\\duelok-dataobject";
    else
        NoDuel_MinimapButton_Icon:SetTexture("Interface\\Addons\\NoDuel\\icons\\duelnok");
        dataobj.icon = "Interface\\Addons\\NoDuel\\icons\\duelnok-dataobject";
    end
end

function NoDuel_MouseOver(button)
    NoDuel_ShowTooltip(button);
end

function NoDuel_MouseOut(button)
    GameTooltip:FadeOut();
end

function NoDuel_ShowTooltip(button)
    GameTooltip_SetDefaultAnchor(GameTooltip, button);
    NoDuel_CreateTooltip(GameTooltip);
end

function NoDuel_CreateTooltip(tooltip)
    tooltip:SetText("|cffffffffNoDuel|r");

    tooltip:AddLine(NODUEL_TOGGLEINFO, nil, nil, nil, 1);
    tooltip:AddLine(NODUEL_WHISPERINFO, nil, nil, nil, 1);

    tooltip:Show();
end
